@@ -34,6 +34,8 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
                         DeprecationWarning)
 
 import apt_pkg
+import distro_info
+import json
 import os
 import subprocess
 import sys
@@ -60,6 +62,7 @@ from .Core.AlertWatcher import AlertWatcher
 from .Core.MyCache import MyCache
 from .Core.roam import NetworkManagerHelper
 from .Core.UpdateList import UpdateList
+from .Core.utils import SoftwarePropertiesPage
 from .backend import (InstallBackend,
                       get_backend)
 
@@ -80,6 +83,7 @@ class UpdateManager(Gtk.Window):
         self.unity = UnitySupport()
         self.controller = None
         self.cache = None
+        self.ua_security_packages = []
         self.update_list = None
         self.meta_release = None
 
@@ -182,15 +186,14 @@ class UpdateManager(Gtk.Window):
         self._start_pane(None)
         sys.exit(0)
 
-    def show_settings(self):
+    def show_settings(self, page_number=SoftwarePropertiesPage.updates):
         try:
             apt_pkg.pkgsystem_unlock()
         except SystemError:
             pass
         cmd = ["/usr/bin/software-properties-gtk",
-               "--open-tab", "2",
-               "--toplevel", "%s" % self.get_window().get_xid()
-               ]
+               "--toplevel", "%s" % self.get_window().get_xid(),
+               "--open-tab", str(page_number)]
         self._look_busy()
         try:
             p = subprocess.Popen(cmd)
@@ -225,20 +228,35 @@ class UpdateManager(Gtk.Window):
                                          cancelled_update, error_occurred)
         self._start_pane(pane)
 
+    def _get_ua_security_status(self):
+        self.ua_security_packages = []
+        try:
+            p = subprocess.run(['pro', 'security-status', '--format=json'], stdout=subprocess.PIPE)
+        except OSError:
+            pass
+        else:
+            s = json.loads(p.stdout.decode('utf-8'))
+            for package in s.get('packages', []):
+                if package.get('service_name', '') == 'standard-security':
+                    continue
+                status = package.get('status', '')
+                if (
+                    status == 'pending_attach'
+                    or status == 'pending_enable'
+                    or status == 'upgrade_available'
+                ):
+                    name = package.get('package', '')
+                    version = package.get('version', '')
+                    size = package.get('download_size', 0)
+                    downloadable = status == 'upgrade_available'
+                    self.ua_security_packages.append(
+                        (name, version, size, downloadable)
+                    )
+            self.cache.create_pro_cache(self.ua_security_packages)
+
     def _make_available_pane(self, install_count, need_reboot=False,
                              cancelled_update=False, error_occurred=False):
-        if install_count == 0:
-            # Need Restart > New Release > No Updates
-            if need_reboot:
-                return NeedRestartDialog(self)
-            pane = self._check_meta_release()
-            if pane:
-                return pane
-            elif cancelled_update:
-                return StoppedUpdatesDialog(self)
-            else:
-                return NoUpdatesDialog(self, error_occurred=error_occurred)
-        else:
+        if install_count != 0 or len(self.ua_security_packages) > 0:
             header = None
             desc = None
             if error_occurred:
@@ -248,6 +266,19 @@ class UpdateManager(Gtk.Window):
                 desc = _("Updated software is available from "
                          "a previous check.")
             return UpdatesAvailable(self, header, desc, need_reboot)
+        else:
+            # Need Restart > New Release > No Updates
+            if need_reboot:
+                return NeedRestartDialog(self)
+            dist_upgrade = self._check_meta_release()
+            if dist_upgrade:
+                return dist_upgrade
+            elif cancelled_update:
+                return StoppedUpdatesDialog(self)
+            elif self.hwe_replacement_packages:
+                return HWEUpgradeDialog(self)
+            else:
+                return NoUpdatesDialog(self, error_occurred=error_occurred)
 
     def start_error(self, update_and_retry, header, desc):
         if update_and_retry:
@@ -344,9 +375,13 @@ class UpdateManager(Gtk.Window):
                 Gtk.main_iteration()
         iterate()
 
+        self._get_ua_security_status()
+
         self.update_list = UpdateList(self)
         try:
-            self.update_list.update(self.cache, eventloop_callback=iterate)
+            self.update_list.update(self.cache, eventloop_callback=iterate,
+                                    ua_security_packages=self.
+                                    ua_security_packages)
         except SystemError as e:
             header = _("Could not calculate the upgrade")
             desc = _("An unresolvable problem occurred while "
