@@ -46,10 +46,16 @@ class InstallBackendAptdaemon(InstallBackend, BuilderDialog):
         self.unity = UnitySupport()
         self._expanded_size = None
         self.button_cancel = None
+        self.trans_failed_msg = None
 
     def close(self):
-        if self.button_cancel:
-            self.button_cancel.clicked()
+        if self.button_cancel and self.button_cancel.get_sensitive():
+            try:
+                self.button_cancel.clicked()
+            except Exception:
+                # there is not much left to do if the transaction can't be
+                # canceled
+                pass
             return True
         else:
             return False
@@ -76,16 +82,16 @@ class InstallBackendAptdaemon(InstallBackend, BuilderDialog):
             raise
 
     @inline_callbacks
-    def commit(self, pkgs_install, pkgs_upgrade):
+    def commit(self, pkgs_install, pkgs_upgrade, pkgs_remove=[]):
         """Commit a list of package adds and removes"""
         try:
             apt.apt_pkg.pkgsystem_unlock()
         except SystemError:
             pass
         try:
-            reinstall = remove = purge = downgrade = []
+            reinstall = purge = downgrade = []
             trans = yield self.client.commit_packages(
-                pkgs_install, reinstall, remove, purge, pkgs_upgrade,
+                pkgs_install, reinstall, pkgs_remove, purge, pkgs_upgrade,
                 downgrade, defer=True)
             trans.connect("progress-changed", self._on_progress_changed)
             yield self._show_transaction(trans, self.ACTION_INSTALL,
@@ -94,6 +100,8 @@ class InstallBackendAptdaemon(InstallBackend, BuilderDialog):
             self._action_done(self.ACTION_INSTALL,
                               authorized=False, success=False,
                               error_string=None, error_desc=None)
+        except errors.TransactionFailed as e:
+            self.trans_failed_msg = str(e)
         except dbus.DBusException as e:
             #print(e, e.get_dbus_name())
             if e.get_dbus_name() != "org.freedesktop.DBus.Error.NoReply":
@@ -223,15 +231,28 @@ class InstallBackendAptdaemon(InstallBackend, BuilderDialog):
     def _on_finished(self, trans, status, action):
         error_string = None
         error_desc = None
+        trans_failed = False
         if status == EXIT_FAILED:
             error_string = get_error_string_from_enum(trans.error.code)
             error_desc = get_error_description_from_enum(trans.error.code)
+            if self.trans_failed_msg:
+                trans_failed = True
+                error_desc = error_desc + "\n" + self.trans_failed_msg
         # tell unity to hide the progress again
         self.unity.set_progress(-1)
         is_success = (status == EXIT_SUCCESS)
-        self._action_done(action,
-                          authorized=True, success=is_success,
-                          error_string=error_string, error_desc=error_desc)
+        try:
+            self._action_done(action,
+                              authorized=True, success=is_success,
+                              error_string=error_string, error_desc=error_desc,
+                              trans_failed=trans_failed)
+        except TypeError:
+            # this module used to be be lazily imported and in older code
+            # trans_failed= is not accepted
+            # TODO: this workaround can be dropped in Ubuntu 20.10
+            self._action_done(action,
+                              authorized=True, success=is_success,
+                              error_string=error_string, error_desc=error_desc)
 
 
 if __name__ == "__main__":
@@ -243,5 +264,5 @@ if __name__ == "__main__":
     app = UpdateManager(data_dir, options)
 
     b = InstallBackendAptdaemon(app, None)
-    b.commit(["2vcard"], [])
+    b.commit(["2vcard"], [], [])
     Gtk.main()
